unit ICFontWhats;

interface

	uses
		ICWindowGlobals;

	function WhatOpenFont (wt: WindowType; item: integer): OSErr;
	function WhatFlushFont (wt: WindowType; item: integer): OSErr;
	function WhatClickFont (wt: WindowType; item: integer; er: eventRecord): OSErr;
	function WhatClickFontSize (wt: WindowType; item: integer; er: eventRecord): OSErr;

implementation

	uses
		Script, 

		ICStrH, 

		ICKeys, ICAPI, ICDialogs, ICMiscSubs, ICSubs, ICDocUtils;

	procedure SetSizeOutlines (sizemenu: MenuHandle; font_num: integer);
		var
			tmpstr: Str255;
			i: integer;
			the_size: longint;
	begin
		for i := 1 to CountMItems(sizemenu) do begin
			GetItem(sizemenu, i, tmpstr);
			StringToNum(tmpstr, the_size);
			if RealFont(font_num, the_size) then begin
				SetItemStyle(sizemenu, i, [outline]);
			end
			else begin
				SetItemStyle(sizemenu, i, []);
			end; (* if *)
		end; (* for *)
	end; (* SetSizeOutlines *)

	procedure GetCurrentSetting (dlg: DialogPtr; item: integer; var font_setting: ICFontRecord; var font_num: integer);
		var
			fontmenu, sizemenu: MenuHandle;
			tmpstr: Str255;
			tmpint: longint;
	begin
		fontmenu := GetPopupMHandle(dlg, item);
		sizemenu := GetPopupMHandle(dlg, item + 1);
		(* collect current values *)
		font_setting.face := [];
		GetItem(fontmenu, GetDCtlValue(dlg, item), font_setting.font);
		GetItem(sizemenu, GetDCtlValue(dlg, item + 1), tmpstr);
		StringToNum(tmpstr, tmpint);
		if (tmpint > 0) and (tmpint < 32767) then begin
			font_setting.size := tmpint;
		end
		else begin
			font_setting.size := GetDefFontSize;
		end; (* if *)
		GetFNum(font_setting.font, font_num);
	end;

	procedure DisplayFontProc (dlg: DialogPtr; item: integer);
		var
			font_setting: ICFontRecord;
			saved: SavedWindowInfo;
			font: integer;
			r: rect;
			s: str255;
			name: Str255;
	begin
		SetPort(dlg);
		GetCurrentSetting(dlg, item - 2, font_setting, font);
		EnterWindow(dlg, font, font_setting.size, [], saved);
		GetDItemRect(dlg, item, r);
		PenPat(gray);
		FrameRect(r);
		PenPat(black);
		GetFontName(font, name);
		if (name = 'Chicago') & (((item = 3) & (font_setting.size <> 12)) | (item = 6)) then begin
			s := GetIndStr(128, 16);
		end
		else begin
			s := GetIndStr(128, 14);
		end; (* if *)
		InsetRect(r, 3, 3);
		TextBox(@s[1], length(s), r, teJustLeft);
		ExitWindow(saved);
	end;

	function WhatOpenFont (wt: WindowType; item: integer): OSErr;
		var
			dlg: WindowPtr;
			fontmenu: MenuHandle;
			sizemenu: MenuHandle;
			font_setting: ICFontRecord;
			attr: longint;
			i: integer;
			tmpstr: Str255;
			test_name: Str255;
			count: longint;
			font_num: integer;
	begin
		dlg := windowinfo[wt].window;
		count := sizeof(font_setting);
		if (ICMapErr(ICGetPref(GetInstance, windowinfo[wt].items[item]^.key, attr, @font_setting, count)) <> noErr) | (count <> sizeof(font_setting)) then begin
			GetFontName(GetAppFont, font_setting.font);
			font_setting.face := [];
			font_setting.size := GetDefFontSize;
		end; (* if *)
		ProcessAttributes(wt, item, attr);
		GetFNum(font_setting.font, font_num);
		if font_num = 0 then begin
			GetFontName(GetAppFont, font_setting.font);
			GetFNum(font_setting.font, font_num);
		end; (* if *)
		fontmenu := GetPopupMHandle(dlg, item);
		for i := 1 to CountMItems(fontmenu) do begin
			GetItem(fontmenu, i, test_name);
			if IUEqualString(test_name, font_setting.font) = 0 then begin
				SetDCtlValue(dlg, item, i);
			end; (* if *)
		end; (* for *)
		NumToString(font_setting.size, tmpstr);
		sizemenu := GetPopupMHandle(dlg, item + 1);
		for i := 1 to CountMItems(sizemenu) do begin
			GetItem(sizemenu, i, test_name);
			if IUEqualString(test_name, tmpstr) = 0 then begin
				SetDCtlValue(dlg, item + 1, i);
			end; (* if *)
		end; (* for *)
		SetSizeOutlines(sizemenu, font_num);
		SetDCtlEnable(dlg, item, not IsLocked(wt, item));
		SetDCtlEnable(dlg, item + 1, not IsLocked(wt, item));
		SetDItemHandle(dlg, item + 2, handle(@DisplayFontProc));
		WhatOpenFont := noErr;
	end; (* WhatOpenFont *)

	function WhatClickFont (wt: WindowType; item: integer; er: eventRecord): OSErr;
		var
			dlg: DialogPtr;
			font_setting: ICFontRecord;
			font_num: integer;
	begin
		dlg := windowinfo[wt].window;
		GetCurrentSetting(dlg, item, font_setting, font_num);
		SetSizeOutlines(GetPopupMHandle(dlg, item + 1), font_num);
		DisplayFontProc(dlg, item + 2);
		WhatClickFont := noErr;
	end;  (* WhatClickFont *)

	function WhatClickFontSize (wt: WindowType; item: integer; er: eventRecord): OSErr;
	begin
		DisplayFontProc(windowinfo[wt].window, item + 1);
		WhatClickFontSize := noErr;
	end;

	function WhatFlushFont (wt: WindowType; item: integer): OSErr;
		var
			dlg: DialogPtr;
			err: OSErr;
			font_setting: ICFontRecord;
			old_font_setting: ICFontRecord;
			attr: longint;
			count: longint;
			font: integer;
	begin
		dlg := windowinfo[wt].window;
		GetCurrentSetting(dlg, item, font_setting, font);
		(* get old values *)
		count := sizeof(old_font_setting);
		if (ICMapErr(ICGetPref(GetInstance, windowinfo[wt].items[item]^.key, attr, @old_font_setting, count)) <> noErr) | (count <> sizeof(old_font_setting)) then begin
			old_font_setting.font := '';
		end; (* if *)
		(* write it back if changed *)
		err := noErr;
		if (old_font_setting.size <> font_setting.size) or (old_font_setting.face <> font_setting.face) or (IUEqualString(old_font_setting.font, font_setting.font) <> 0) then begin
			if not IsLocked(wt, item) then begin
				err := ICMapErr(ICSetPref(GetInstance, windowinfo[wt].items[item]^.key, ICattr_no_change, @font_setting, sizeof(font_setting)));
				if err = noErr then begin
					DirtyDocument;
				end; (* if *)
			end; (* if *)
		end; (* if *)
		WhatFlushFont := err;
	end; (* WhatFlushFont *)

end. (* ICFontWhats *)