/* FilterNull.c */
/*****************************************************************************/
/*                                                                           */
/*    Out Of Phase:  Digital Music Synthesis on General Purpose Computers    */
/*    Copyright (C) 1994  Thomas R. Lawrence                                 */
/*                                                                           */
/*    This program is free software; you can redistribute it and/or modify   */
/*    it under the terms of the GNU General Public License as published by   */
/*    the Free Software Foundation; either version 2 of the License, or      */
/*    (at your option) any later version.                                    */
/*                                                                           */
/*    This program is distributed in the hope that it will be useful,        */
/*    but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/*    GNU General Public License for more details.                           */
/*                                                                           */
/*    You should have received a copy of the GNU General Public License      */
/*    along with this program; if not, write to the Free Software            */
/*    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              */
/*                                                                           */
/*    Thomas R. Lawrence can be reached at tomlaw@world.std.com.             */
/*                                                                           */
/*****************************************************************************/

/* Based on material from pages 184-190 of */
/* Dodge, Charles and Jerse, Thomas A. */
/* Computer Music:  Synthesis, Composition, and Performance */
/* Schirmer Books, New York, 1985 */

#include "MiscInfo.h"
#include "Audit.h"
#include "Debug.h"
#include "Definitions.h"

#include "FilterNull.h"
#include "Memory.h"


struct FilterNullRec
	{
		/* link */
		FilterNullRec*			Next;
	};


static FilterNullRec*			FreeList = NIL;


/* flush free list */
void											FlushCachedFilterNullStuff(void)
	{
		while (FreeList != NIL)
			{
				FilterNullRec*			Temp;

				Temp = FreeList;
				FreeList = FreeList->Next;
				ReleasePtr((char*)Temp);
			}
	}


/* create a new filter record */
FilterNullRec*						NewFilterNull(void)
	{
		FilterNullRec*					Filter;

		if (FreeList != NIL)
			{
				Filter = FreeList;
				FreeList = FreeList->Next;
			}
		 else
			{
				Filter = (FilterNullRec*)AllocPtrCanFail(sizeof(FilterNullRec),"FilterNullRec");
				if (Filter == NIL)
					{
						return NIL;
					}
			}
		return Filter;
	}


/* dispose filter record */
void											DisposeFilterNull(FilterNullRec* Filter)
	{
		CheckPtrExistence(Filter);
		Filter->Next = FreeList;
		FreeList = Filter;
	}


/* apply filter to a sample value */
float											ApplyFilterNull(FilterNullRec* Filter, float Xin)
	{
		CheckPtrExistence(Filter);
		return Xin;
	}
